#include "SudokuSolver.h"

SudokuSolver::SudokuSolver(const SudokuTask & t):task(t),current(NULL),record_solve(false), need_record_solve(false), analysed(false), step_limit(0), count_of_choices(0), random_mode(false)
{
}

SudokuSolver::~SudokuSolver()
{
   if ( current != NULL ){ delete current; }
};

void SudokuSolver::SetStepLimit(int limit)
{
   step_limit = limit;
};
void SudokuSolver::SetRandomSolveMode(bool mode)
{
   random_mode = mode;
}

int SudokuSolver::Solve()
{
   if (current) delete current;
   desk_history.clear();
   choices.clear();
   visible_history.clear();
   excess_choices.clear();
   solves.clear();
   analysed = false;
   
   current = new SudokuDesk(task.SmallSize());
   current->SetObserver(this);
   record_solve = false;
   if (need_record_solve) record_solve = true;

   
   int i, j;
   for(i=0;i<task.Size();i++) for(j=0;j<task.Size();j++)
   {
      if ( task.Get(i, j) > 0)
      {
         bool new_informaton;
         current->SetItem(i, j, task.Get(i, j), new_informaton);
         if (!new_informaton)
         {
            excess_choices.push_back(SudokuChoice( i, j, task.Get(i, j)));
         }
      }
   }
   //Easy and medium sudoku is solved at this moment! :)

   if (!solve_sudoku()){
      analysed = true;
      has_solve = false;
      all_solves_found = true;
      if (step_limit > 0 && step_count >= step_limit) return -1;
      return 0;
   }
   record_solve = false;
   solves.push_back(*current);
   analysed = true;
   has_solve = true;
   all_solves_found = false;

   if (!solve_another())
   { 
      if (step_limit > 0 && step_count >= step_limit) return -1;
      all_solves_found = true;
      return 1;
   }else
   {
      if (step_limit > 0 && step_count >= step_limit) return -1;
      solves.push_back(*current);
      return 2;
   }
   
};    

const std::list<SudokuDesk> & SudokuSolver::FindAllSolutions()
{
   if (!analysed)
   {
      Solve();
   }
   if (!all_solves_found)
   {
      while (solve_another())
      {
         if (step_limit > 0 && step_count >= step_limit) throw SudokuException(QT_TR_NOOP("All Solution can not by found because of max step count reached"));;                  
         solves.push_back(*current);
      }
   }
   all_solves_found = true;
   return solves;
};

const std::list<SudokuDesk> & SudokuSolver::GetSolutions()
{
   return solves;
};

bool SudokuSolver::solve_sudoku()
{
   step_count = 0;
   if ( current->IsSolved()) return true;
   while(true){
      if (!solve_sudoku_step()) return false;
      if (current->IsSolved()) return true;
   }
}

bool SudokuSolver::solve_another()
{
   if ( desk_history.empty() ) return false;
   step_count = 0;
   *current = desk_history.back();
   desk_history.pop_back();
   SudokuChoice c = choices.back();
   choices.pop_back();
   current->ExcludeItem(c.i, c.j, c.value);
   return solve_sudoku();
}

bool SudokuSolver::solve_sudoku_step()
{
   step_count++;
   if (step_limit > 0 && step_count >= step_limit){ return false; };
   if ( step_count > 1000 ) record_solve = false;
   
   SudokuChoice c;
   if ( current->Unsoluble() )
   {
      //If solution is impossible, get previous state from stack and mark previous choice as wrong
      if ( desk_history.empty() ) return false;

      if (record_solve)
      {   

         SudokuDesk * bad_desk = current;
         SudokuDesk * good_desk = &(desk_history.back());         
         
         for(int i=0; i<current->GetSize();i++)for(int j=0; j<current->GetSize();j++)
         {
            if (good_desk->GetItem(i, j)==0 && bad_desk->GetItem(i, j) != 0)
            {
               visible_history.push_back(SudokuSolveStep(i, j, bad_desk->GetItem(i, j), unknown));               
            }
         }
      }  
      *current = desk_history.back();
      
      desk_history.pop_back();
      c = choices.back();
      choices.pop_back();
      
      if (record_solve)
      {
         visible_history.push_back(SudokuSolveStep(c.i, c.j, c.value, aborted));
      }
      current->ExcludeItem(c.i, c.j, c.value);

      return true;
   }

   if (!current->GetMinVaryItem(c.i, c.j)) return true;      

   if (random_mode)
   {
      // Get random possible value
      c.value= current->GetOneOfItems(c.i, c.j);      
   }else
   {
      //Get best of possible values
      c.value= current->GetBestOneOfItems(c.i, c.j);      
   }
   
   //Push state of desk and our choice to stack
   SudokuDesk s = *current;
   desk_history.push_back(s);
   choices.push_back(c);
   if (record_solve)
   {
      visible_history.push_back(SudokuSolveStep(c.i, c.j, c.value, questionable));
      count_of_choices++;
   }

   //Allpy choice
   current->SetItem(c.i, c.j, c.value);
   
   return true;
};    

void SudokuSolver::NextElement(int i, int j, int value, SudokuStepType type){
   if (record_solve)
   {
      visible_history.push_back(SudokuSolveStep(i, j, value, type));
   }
};

SudokuDesk SudokuSolver::GetFirstSolve()
{
   if (analysed)
   {
      if (has_solve)
      {
         return solves.front();
      }else{
         throw SudokuException(QT_TR_NOOP("Solution does not exists"));
      }
   }else{
      throw SudokuException(QT_TR_NOOP("Task was not solved"));
   }
}

SudokuDesk SudokuSolver::GetSecondSolve()
{
   if (analysed)
   {
      if (has_solve)
      {
         if ( solves.size() > 1 )
         {

            return *(++solves.begin());
         }else
         {
            throw SudokuException(QT_TR_NOOP("Second Solution does not exists"));
         }
      }else{
         throw SudokuException(QT_TR_NOOP("Solution does not exists"));
      }
   }else{
      throw SudokuException(QT_TR_NOOP("Task was not solved"));
   }
}
