/****************************************************************************

    QSudoku - a application for solving and generating sudoku puzzles
    Copyright (C) 2005  Yulia Shabunio

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

 ****************************************************************************/

#ifndef SUDOKU_SOLVER
#define SUDOKU_SOLVER 1

#include "Tools.h"
#include "SudokuDesk.h"
#include "SudokuTask.h"

struct SudokuChoice
{
  int i;
  int j;
  int value;
  SudokuChoice():i(-1), j(-1), value(0){};   
  SudokuChoice( int i, int j, int value ):i(i), j(j), value(value){}; 
};

struct SudokuSolveStep
{
   int i;
   int j;
   int value;
   SudokuSolveObserver::SudokuStepType type;
   SudokuSolveStep( int i, int j, int value, SudokuSolveObserver::SudokuStepType type):i(i),j(j),value(value),type(type){};
};   

class SudokuSolver : public SudokuSolveObserver
{
private:

   SudokuTask task;
   std::list<SudokuDesk> solves;

   SudokuDesk * current;

   std::list<SudokuDesk> desk_history;
   std::list<SudokuChoice> choices;
   std::list<SudokuSolveStep> visible_history;
   std::list<SudokuChoice> excess_choices;

   bool record_solve;
   bool need_record_solve;

   bool analysed;
   bool has_solve;
   bool all_solves_found;

   int step_count;
   int step_limit;
   int count_of_choices;
   bool random_mode;
   
   bool solve_sudoku();
   bool solve_another();
   bool solve_sudoku_step();   
public:
   //constructor
   SudokuSolver(const SudokuTask & t);
   virtual ~SudokuSolver();
   
   //Solve sudoku task
   //Return values:
   //0 - where is not solve, 1 - solve one and only one, 2 - more than one solve. -1 - limit count of solve step is reached.
   int Solve(); 
   //return desk of first solution. Throws exception, if where is no solution yet.
   SudokuDesk GetFirstSolve();
   //return desk of second solution. Throws exception, if where is no second solution yet.
   SudokuDesk GetSecondSolve();
   //Get all solutions of task, known at this moment.
   const std::list<SudokuDesk> & GetSolutions();
   
   
   //Find and get all solutions of task. May by very time expensive.
   const std::list<SudokuDesk> & FindAllSolutions();
   //Get task elemens, that naturally follow from other elements and can be removed with impunity
   const std::list<SudokuChoice> & GetExcessItems(){return excess_choices;};
   //Get history of receiving first solution, item-by-item. Generated only if SaveSolve() was called
   const std::list<SudokuSolveStep> & GetSolveHistory(){return visible_history;};
   //Get count of choices, that was made in the solve process. Generated only if SaveSolve() was called
   int GetCountOfChoices(){return count_of_choices;};

   //Instruction to save solution history
   void SaveSolve(){ need_record_solve = true; };

   //Callback function for solved desk to save solution history.
   //It is not allowed to call it from GUI
   virtual void NextElement(int i, int j, int value, SudokuStepType type);

   //Setting max limit of solve step count.
   //Useful for generation new sudoku, where interesting fast solving, but don't need excact solutions.
   //If solution reach limit steps, it returns -1
   // 0 means infinity
   void SetStepLimit(int limit);
   //If mode, solver will do random guesses.
   //If !mode, solber will try to get best guess.
   void SetRandomSolveMode(bool mode);
};
     
      
#endif
